
#include <string>
#include <stdexcept>

#if defined(__GL_AUTOCHECK) && (defined(_DEBUG) || defined(__GL_FORCE_AUTOCHECK))

/**
 *  @def __HAVE_GL_AUTOCHECK
 *  @brief defined in case GL_AUTOCHECK_BLOCK is not empty
 */
#define __HAVE_GL_AUTOCHECK

#ifndef FIX_GL_FUNCTION_LINKING_CONFLICTS
#define __ConCat__(a,b) a##b
#endif // !FIX_GL_FUNCTION_LINKING_CONFLICTS

/**
 *  @def GLAutoCheckFuncPtrName
 *  @brief creates name for a _glachwrap function
 *  @param[in] fn is a name of an OpenGL function
 */
#define GLAutoCheckFuncPtrName(fn) __ConCat__(fn,_glachwrap)

/**
 *  @def GLAutoCheckExtraParams
 *  @brief additional parameters that are needed for the autocheck functions
 */
#define GLAutoCheckExtraParams , __FILE__, __LINE__

/**
 *  @def GLAutoCheckExtraParamsSA
 *  @brief standalone parameters that are needed for the autocheck functions
 */
#define GLAutoCheckExtraParamsSA __FILE__, __LINE__

/**
 *  @def GLAutoCheckFire
 *  @brief OpenGL autocheck error event handler
 *  @param[in] what is exception description
 */
#if defined(__GL_AUTOCHECK_USE_EXCEPTION)
#define GLAutoCheckFire(what) throw std::runtime_error(what)
#elif defined(__GL_AUTOCHECK_USE_BREAKPOINT)
#define GLAutoCheckFire(what) do { fprintf(stderr, "GL_AUTOCHECK: %s\n", (what)); __asm { int 3 } } while(0)
#elif defined(__GL_AUTOCHECK_USE_ABORT)
#define GLAutoCheckFire(what) do { fprintf(stderr, "GL_AUTOCHECK: %s\n", (what)); abort(); } while(0)
#else // if defined(__GL_AUTOCHECK_USE_EXIT)
#define GLAutoCheckFire(what) do { fprintf(stderr, "GL_AUTOCHECK: %s\n", (what)); exit(-1); } while(0)
#endif // __GL_AUTOCHECK_USE_EXCEPTION

/**
 *  @def GL_AUTOCHECK_BLOCK
 *
 *  @param[in] p_s_function is the OpenGL function being called
 *  @param[in] p_s_file is name of file of invocation
 *  @param[in] n_line is (one-based) index of source code line of invocation
 *
 *  @brief checks for OpenGL errors and throws std::runtime_error
 *      with error descritption in case any error occured
 *  @note This is only available in debug mode unless __GL_FORCE_AUTOCHECK is defined.
 */
#if defined(__STL_UTILS_INCLUDED)

#define GL_AUTOCHECK_BLOCK(p_s_function,p_s_file,n_line) \
    do { \
        GLenum n_error; \
        if((n_error = glGetError()) == GL_NO_ERROR) \
            break;  \
        std::string s_message; \
        switch(n_error) { \
        case GL_INVALID_ENUM: \
            stl_ut::Format(s_message, "error: file \'" "%s" "\', line %d, function %s()" \
                ": OpenGL error: GL_INVALID_ENUM", p_s_file, n_line, p_s_function); \
            GLAutoCheckFire(s_message.c_str()); \
        case GL_INVALID_VALUE: \
            stl_ut::Format(s_message, "error: file \'" "%s" "\', line %d, function %s()" \
                ": OpenGL error: GL_INVALID_VALUE", p_s_file, n_line, p_s_function); \
            GLAutoCheckFire(s_message.c_str()); \
        case GL_INVALID_OPERATION: \
            stl_ut::Format(s_message, "error: file \'" "%s" "\', line %d, function %s()" \
                ": OpenGL error: GL_INVALID_OPERATION", p_s_file, n_line, p_s_function); \
            GLAutoCheckFire(s_message.c_str()); \
        case GL_OUT_OF_MEMORY: \
            stl_ut::Format(s_message, "error: file \'" "%s" "\', line %d, function %s()" \
                ": OpenGL error: GL_OUT_OF_MEMORY", p_s_file, n_line, p_s_function); \
            GLAutoCheckFire(s_message.c_str()); \
        default: \
            stl_ut::Format(s_message, "error: file \'" "%s" "\', line %d, function %s()" \
                ": OpenGL error: 0x%04x", p_s_file, n_line, p_s_function, n_error); /* this shouldn't really happen */ \
            GLAutoCheckFire(s_message.c_str()); \
        }; \
    } while(0)
// use stl utils' Format()

#elif defined(_MSC_VER) && !defined(__MWERKS__) && _MSC_VER >= 1400

#define GL_AUTOCHECK_BLOCK(p_s_function,p_s_file,n_line) \
    do { \
        GLenum n_error; \
        if((n_error = glGetError()) != GL_NO_ERROR) { \
            char p_s_buffer[16]; \
            const char *p_s_error; \
            switch(n_error) { \
            case GL_INVALID_ENUM: \
                p_s_error = "GL_INVALID_ENUM"; \
                break; \
            case GL_INVALID_VALUE: \
                p_s_error = "GL_INVALID_VALUE"; \
                break; \
            case GL_INVALID_OPERATION: \
                p_s_error = "GL_INVALID_OPERATION"; \
                break; \
            case GL_OUT_OF_MEMORY: \
                p_s_error = "GL_OUT_OF_MEMORY"; \
                break; \
            default: \
                sprintf(p_s_buffer, "0x%04x", n_error); /* this shouldn't really happen */ \
                p_s_error = p_s_buffer; \
            }; \
            const char *p_s_fmt = "error: file \'%s\', line %d, " \
                "function %s(): OpenGL error: %s"; \
            std::string s_message; \
            s_message.resize(strlen(p_s_fmt) + strlen(p_s_file) + \
                16 + strlen(p_s_function) + strlen(p_s_error)); \
            _snprintf(&s_message[0], s_message.length() * sizeof(char), \
                p_s_fmt, p_s_file, n_line, p_s_function, p_s_error); \
            GLAutoCheckFire(s_message.c_str()); \
        } \
    } while(0)
// use snprintf()

#else // _MSC_VER && !__MWERKS__ && _MSC_VER >= 1400

#define GL_AUTOCHECK_BLOCK(p_s_function,p_s_file,n_line) \
    do { \
        GLenum n_error; \
        if((n_error = glGetError()) != GL_NO_ERROR) { \
            char p_s_buffer[16]; \
            const char *p_s_error; \
            switch(n_error) { \
            case GL_INVALID_ENUM: \
                p_s_error = "GL_INVALID_ENUM"; \
                break; \
            case GL_INVALID_VALUE: \
                p_s_error = "GL_INVALID_VALUE"; \
                break; \
            case GL_INVALID_OPERATION: \
                p_s_error = "GL_INVALID_OPERATION"; \
                break; \
            case GL_OUT_OF_MEMORY: \
                p_s_error = "GL_OUT_OF_MEMORY"; \
                break; \
            default: \
                sprintf(p_s_buffer, "0x%04x", n_error); /* this shouldn't really happen */ \
                p_s_error = p_s_buffer; \
            }; \
            const char *p_s_fmt = "error: file \'%s\', line %d, " \
                "function %s(): OpenGL error: %s"; \
            std::string s_message; \
            s_message.resize(strlen(p_s_fmt) + strlen(p_s_file) + \
                16 + strlen(p_s_function) + strlen(p_s_error)); \
            sprintf(&s_message[0], p_s_fmt, p_s_file, n_line, p_s_function, p_s_error); \
            GLAutoCheckFire(s_message.c_str()); \
        } \
    } while(0)
// use unsafe sprintf()

#endif // _MSC_VER && !__MWERKS__ && _MSC_VER >= 1400

#else // __GL_AUTOCHECK && (_DEBUG || __GL_FORCE_AUTOCHECK)

/**
 *  @def GLAutoCheckFuncPtrName
 *  @brief creates name for a _glachwrap function
 *  @param[in] fn is a name of an OpenGL function
 */
#define GLAutoCheckFuncPtrName(fn) GLFuncPtrName(fn)

/**
 *  @def GLAutoCheckExtraParams
 *  @brief additional parameters that are needed for the autocheck functions
 */
#define GLAutoCheckExtraParams 

/**
 *  @def GLAutoCheckExtraParamsSA
 *  @brief standalone parameters that are needed for the autocheck functions
 */
#define GLAutoCheckExtraParamsSA 

/**
 *  @def GL_AUTOCHECK_BLOCK
 *
 *  @param[in] p_s_function is the OpenGL function being called
 *  @param[in] p_s_file is name of file of invocation
 *  @param[in] n_line is (one-based) index of source code line of invocation
 *
 *  @brief has no effect (_DEBUG or __GL_FORCE_AUTOCHECK are not defined).
 */
#define GL_AUTOCHECK_BLOCK(p_s_function,p_s_file,n_line) do {} while(0)

#endif // __GL_AUTOCHECK && (_DEBUG || __GL_FORCE_AUTOCHECK)

