 *  @author -tHE SWINe-
 *  @brief OpenGL 4.0 and extensions entry points
 *
 *  This header contains entry points of relevant OpenGL 4 functions / extensions.
 *      It was prepared from gl3.h, available from &lt;http://www.opengl.org&gt;,
 *      from extension registry at &lt;http://www.opengl.org&gt; and from OpenGL function
 *      documentation pages at &lt;http://www.opengl.org/sdk/docs/man4/xhtml/&gt;.
 *
 *  The difference between OpenGL30.h and OpenGL40.h is OpenGL40.h also includes
 *      OpenCL headers to support OpenCL interop (GL_ARB_cl_event).
 *
 *  @date 2009-11-28
 *
 *  core functionality / core extensions:<br>
 *      OpenGL 1.0<br>
 *      OpenGL 1.1<br>
 *      OpenGL 1.2<br>
 *      OpenGL 1.3<br>
 *      OpenGL 1.4<br>
 *      OpenGL 1.5<br>
 *      OpenGL 2.0<br>
 *      OpenGL 2.1<br>
 *      OpenGL 3.0
 *          - GL_ARB_depth_buffer_float
 *          - GL_ARB_framebuffer_object
 *          - GL_ARB_framebuffer_sRGB
 *          - GL_ARB_half_float_vertex
 *          - GL_ARB_map_buffer_range
 *          - GL_ARB_texture_compression_rgtc
 *          - GL_ARB_texture_rg
 *          - GL_ARB_vertex_array_object
 *
 *      OpenGL 3.1
 *          - GL_ARB_texture_buffer_object
 *          - GL_ARB_uniform_buffer_object
 *          - GL_ARB_draw_instanced
 *          - GL_ARB_copy_buffer
 *
 *      OpenGL 3.2
 *          - GL_ARB_depth_clamp
 *          - GL_ARB_draw_elements_base_vertex
 *          - GL_ARB_fragment_coord_conventions
 *          - GL_ARB_provoking_vertex
 *          - GL_ARB_seamless_cube_map
 *          - GL_ARB_sync
 *          - GL_ARB_texture_multisample
 *          - GL_ARB_vertex_array_bgra
 *
 *  present extensions:
 *          - GL_EXT_draw_buffers2          (separate blend enables, separate color masks for mutliple draw buffers)
 *          - GL_ARB_draw_buffers_blend     (different blend equations for multiple draw buffers)
 *          - GL_ARB_sample_shading         (enforced shading for multisample antialiassing)
 *          - GL_ARB_texture_cube_map_array (texture array for cube-maps)
 *          - GL_ARB_texture_gather         (new functions for GLSL, returns x components of four sampled texels, lets shader do it's own filtering, instead of bilinear. without GL_EXT_texture_swizzle it's only monochrome though)
 *          - GL_ARB_texture_query_lod      (new functions for GLSL, returns LOD sampler would use for texture (instead of sampled color))
 *          - GL_EXT_timer_query                (new query object target for GPU timing)
 *          - GL_EXT_gpu_program_parameters (single call to fill multiple GPU program parameters; useless because we do not have programs anymore - only shaders)
 *          - GL_EXT_bindable_uniform           (provides bindable buffers to contain uniform variables, so they don't have to be specified)
 *          - GL_ARB_instanced_arrays           (sets divisor to vertex attributes; when set to n = nonzero, a particular attribute advances every n vertices; similar to direct-x function SetStreamFrequency)
 *          - GL_ARB_shader_texture_lod     (additions to GLSL; contains no new tokens for GL)
 *          - GL_ARB_texture_buffer_object  (new texture type, buffer object is now bindable as large 1D texture; it's unclear how exactly is it used)
 *
 *          - GL_S3_s3tc                        (pixel formats)
 *          - GL_EXT_texture_compression_latc   (pixel formats)
 *          - GL_EXT_texture_compression_s3tc   (pixel formats)
 *          - GL_EXT_texture_filter_anisotropic (anisotropic texture filter control)
 *          - GL_EXT_abgr                       (ABGR pixel format)
 *
 *  new extensions which are not present yet:
 *      -   GL_ARB_compatibility - a big one, fork of OpenGL 3.1
 *
 *  removed:
 *          - GL_ARB_texture_compression - promoted to core in OpenGL 1.3
 *          - GL_ARB_vertex_blend - relies on deprecated parts of fixed-function pipeline
 *          - GL_ARB_matrix_palette - relies on deprecated parts of fixed-function pipeline
 *          - GL_EXT_compiled_vertex_array - is now obsolete, extension wasn't updated for a long time
 *          - GL_EXT_vertex_array - is obsolete, became core and in turn, became deprecated
 *          - GL_ARB_vertex_program, GL_ARB_fragment_program - deprecated; only shading language 1.30 is supported
 *          - GL_ARB_vertex_shader, GL_ARB_fragment_shader, GL_ARB_shader_object - became core in OpenGL 2.0
 *          - GL_ARB_vertex_buffer_object - became core in OpenGL 1.5
 *          - GL_ARB_pixel_buffer_object - became core in OpenGL 2.1
 *          - GL_ARB_occlusion_query - became core in OpenGL 1.5
 *          - GL_ARB_multitexture - became core in OpenGL 1.3; n_GetMultitextureFuncPointers() was merged into n_GetGL13FuncPointers()
 *          - GL_EXT_packed_depth_stencil - part of GL_ARB_framebuffer_object, which is core in OpenGL 3.0
 *          - GL_EXT_framebuffer_object - part of GL_ARB_framebuffer_object, which is core in OpenGL 3.0
 *          - GL_EXT_draw_instanced - became core in OpenGL 3.1
 *
 *  not present:
 *          - GL_ARB_imaging (part of OpenGL 1.2 core) - seems to be deprecated
 *          - GL_ARB_color_buffer_float - superseded by OpenGL 3.0
 *          - GL_NV_fence - superseded by GL_ARB_sync
 *          - GL_EXT_texture_integer - superseded by OpenGL 3.0
 *          - GL_EXT_gpu_shader4 - superseded by OpenGL 3.0, OpenGL 3.1
 *          - GL_EXT_geometry_shader4 - superseded by GL_ARB_framebuffer_object, OpenGL 3.2
 *          - GL_EXT_texture_array - superseded by GL_ARB_framebuffer_object, OpenGL 3.0
 *          - GL_EXT_texture_buffer_object - superseded by OpenGL 3.1
 *          - GL_EXT_blend_minmax - superseded by GL_ARB_imaging, OpenGL 1.2
 *          - GL_EXT_blend_subtract - superseded by GL_ARB_imaging, OpenGL 1.2
 *          - GL_ARB_geometry_shader4 - superseded by GL_ARB_framebuffer_object, OpenGL 3.2
 *          - GL_ARB_draw_instanced - directly part of OpenGL 3.1 core
 *          - GL_ARB_half_float_pixel - superseded by GL_ARB_half_float_vertex, part of OpenGL 3.0 core
 *          - GL_ARB_texture_float - part of OpenGL 3.0 core
 *          - GL_ARB_pixel_buffer_object - part of OpenGL 2.1 core
 *
 *  @date 2010-09-19
 *
 *  added the following extensions:
 *          - GL_ARB_shading_language_include
 *          - GL_ARB_texture_compression_bptc
 *          - GL_ARB_blend_func_extended
 *          - GL_ARB_explicit_attrib_location
 *          - GL_ARB_occlusion_query2
 *          - GL_ARB_sampler_objects
 *          - GL_ARB_shader_bit_encoding
 *          - GL_ARB_texture_rgb10_a2ui
 *          - GL_ARB_texture_swizzle
 *          - GL_ARB_timer_query
 *          - GL_ARB_vertex_type_2_10_10_10_rev
 *          - GL_ARB_draw_indirect
 *          - GL_ARB_gpu_shader5
 *          - GL_ARB_gpu_shader_fp64
 *          - GL_ARB_shader_subroutine
 *          - GL_ARB_tessellation_shader
 *          - GL_ARB_texture_buffer_object_rgb32
 *          - GL_ARB_transform_feedback2
 *          - GL_ARB_transform_feedback3
 *          - GL_ARB_es2_compatibility
 *          - GL_ARB_get_program_binary
 *          - GL_ARB_separate_shader_objects
 *          - GL_ARB_shader_precision
 *          - GL_ARB_vertex_attrib64bit
 *          - GL_ARB_viewport_array
 *          - GL_ARB_cl_event
 *          - GL_ARB_debug_output
 *          - GL_ARB_robustness
 *          - GL_ARB_shader_stencil_export
 *
 *  @date 2010-10-29
 *
 *  Unified windows detection macro to "#if defined(_WIN32) || defined(_WIN64)".
 *
 *  @date 2011-07-15
 *
 *  Removed glFramebufferTextureFace(), which was a bug in the documentation. OpenGL 3.2 support
 *  is now reported complete (CGL3ExtensionHandler::n_GetGL32FuncPointers() returns 0,
 *  and not 1 as it did).
 *
 *  Re-organized core functionality (the extensions were all present already):
 *
 *      OpenGL 3.3:
 *          - GL_ARB_shader_bit_encoding
 *          - GL_ARB_blend_func_extended
 *          - GL_ARB_explicit_attrib_location
 *          - GL_ARB_occlusion_query2
 *          - GL_ARB_sampler_objects
 *          - GL_ARB_texture_rgb10_a2ui
 *          - GL_ARB_texture_swizzle
 *          - GL_ARB_timer_query
 *          - GL_ARB_instanced_arrays
 *          - GL_ARB_vertex_type_2_10_10_10_rev
 *
 *      OpenGL 4.0:
 *          - GL_ARB_texture_query_lod
 *          - GL_ARB_draw_indirect
 *          - GL_ARB_gpu_shader5
 *          - GL_ARB_gpu_shader_fp64
 *          - GL_ARB_shader_subroutine
 *          - GL_ARB_tessellation_shader
 *          - GL_ARB_texture_buffer_object_rgb32
 *          - GL_ARB_texture_cube_map_array
 *          - GL_ARB_texture_gather
 *          - GL_ARB_transform_feedback2
 *          - GL_ARB_transform_feedback3
 *
 *      OpenGL 4.1:
 *          - GL_ARB_ES2_compatibility
 *          - GL_ARB_get_program_binary
 *          - GL_ARB_separate_shader_objects
 *          - GL_ARB_shader_precision
 *          - GL_ARB_vertex_attrib_64bit
 *          - GL_ARB_viewport_array
 *
 *  Updated the license information.
 *
 *  @date 2011-07-20
 *
 *  Improved autocheck for statically-linked void functions so if using the user breakpoint,
 *  the code displayed in MSVC is user code (the call that caused the error) and not
 *  OpenGL40.h. This only increases user comfort, the functionality didn't change.
 *
 *  Optimized the code to reduce compilation time.
 *
 *  @date 2011-07-29
 *
 *  Added the "__" prefix to all the switches \#defined or used here.
 *
 *  Added documentation comments to extension guard \#defines and extension __GENERATE_* \#defines.
 *
 *  Added documentation comments to OpenGL enums.
 *
 *  Renamed CGL4ExtensionHandler::b_SupportedWGLExtension() to
 *  CGL4ExtensionHandler::b_SupportedPlatformExtension(). It is now compiled
 *  for both linux and windows.
 *
 *  Removed deprecated tokens.
 *
 *  @date 2011-08-09
 *
 *  Added support for OpenGL 4.2, removed two deprecated tokens.
 *
 *      OpenGL 4.2:
 *          - GL_ARB_texture_compression_bptc
 *          - GL_ARB_compressed_texture_pixel_storage
 *          - GL_ARB_shader_atomic_counters
 *          - GL_ARB_texture_storage
 *          - GL_ARB_transform_feedback_instanced
 *          - GL_ARB_base_instance
 *          - GL_ARB_shader_image_load_store
 *          - GL_ARB_conservative_depth
 *          - GL_ARB_shading_language_420pack
 *          - GL_ARB_internalformat_query
 *          - GL_ARB_map_buffer_alignment
 *          - GL_ARB_shading_language_packing
 *
 *      deprecated:
 *          - GL_NUM_COMPRESSED_TEXTURE_FORMATS
 *          - GL_COMPRESSED_TEXTURE_FORMATS
 *
 *  Also removed some deprecated functions from GL_ARB_robustness.
 *
 *  Added the GL_EXT_direct_state_access extension.
 *
 *  2011-08-15
 *
 *  Added the n_GL_TypeSize() macro, which can be used to determine
 *  OpenGL (internal) datatype size.
 *
 *  Removed some extension features based on the OpenGL deprecation model
 *  (mostly legacy texture formats).
 *
 */

// license from gl3.h
/*
** Copyright (c) 2007-2011 The Khronos Group Inc.
**
** Permission is hereby granted, free of charge, to any person obtaining a
** copy of this software and/or associated documentation files (the
** "Materials"), to deal in the Materials without restriction, including
** without limitation the rights to use, copy, modify, merge, publish,
** distribute, sublicense, and/or sell copies of the Materials, and to
** permit persons to whom the Materials are furnished to do so, subject to
** the following conditions:
**
** The above copyright notice and this permission notice shall be included
** in all copies or substantial portions of the Materials.
**
** THE MATERIALS ARE PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
** MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
** TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
** MATERIALS OR THE USE OR OTHER DEALINGS IN THE MATERIALS.
*/

// license for materials from http://www.opengl.org/sdk/docs/man/xhtml/
/*
** SGI FREE SOFTWARE LICENSE B (Version 2.0, Sept. 18, 2008)
** Copyright (C) 1991-2006 Silicon Graphics, Inc. All Rights Reserved.
** Permission is hereby granted, free of charge, to any person obtaining
** a copy of this software and associated documentation files (the "Software"),
** to deal in the Software without restriction, including without limitation
** the rights to use, copy, modify, merge, publish, distribute, sublicense,
** and/or sell copies of the Software, and to permit persons to whom the Software
** is furnished to do so, subject to the following conditions:
**
** The above copyright notice including the dates of first publication and
** either this permission notice or a reference to http://oss.sgi.com/projects/FreeB/
** shall be included in all copies or substantial portions of the Software.
**
** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
** OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
** FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
** SILICON GRAPHICS, INC. BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
** WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
** OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
** IN THE SOFTWARE.
**
** Except as contained in this notice, the name of Silicon Graphics, Inc. shall
** not be used in advertising or otherwise to promote the sale, use or other
** dealings in this Software without prior written authorization from
** Silicon Graphics, Inc.
*/

