
/*
 *  makes life a little bit easier when including gl.h by mistake
 *
 *  does not include gl.h, defines GL up to 1.1 below, want to prevent gl.h
 *  header inclusion
 */
#if defined(__gl_h_) || defined(__GL_H__) || defined(GL_VERSION_1_1)
#error("warning: gl.h must NOT be included in files using OpenGL40.h")
#endif

/*
 *  makes life a little bit easier when including glext.h by mistake
 *
 *  glext.h is not included anymore, it does contain significant amount of
 *  deprecated functionality, want to prevent glext.h header inclusion
 */
#if defined(__glext_h_) || defined(GL_GLEXT_VERSION)
#error("glext.h must NOT be included in files using OpenGL40.h")
#endif

/**
 *  @def __gl_h_
 *  @brief avoids inclusion of gl.h
 */
#define __gl_h_

/**
 *  @def __GL_H__
 *  @brief avoids inclusion of gl.h
 */
#define __GL_H__

/**
 *  @def __glext_h_
 *  @brief avoids inclusion of glext.h
 */
#define __glext_h_

/**
 *  @def __GLEXT_H__
 *  @brief avoids inclusion of glext.h
 */
#define __GLEXT_H__

/**
 *  @def __GL_NULL_FUNCTION_POINTERS
 *  @brief if defined, initializes entry function pointers to null
 *
 *  In case __GL_NULL_FUNCTION_POINTERS is defined (default), all extension function pointers
 *      are initialized to 0, but user never needs to compare individual function values
 *      so it's posible to save some space in executable file by skipping the initialization.
 */
#define __GL_NULL_FUNCTION_POINTERS

/**
 *  @def __GL_FIX_FUNCTION_LINKING_CONFLICTS
 *  @brief avoids function linking conflicts by renaming function pointers, and fixing
 *      names using \#define
 *
 *  Some compilers may contain newer OpenGL libraries than this header requires
 *      (OpenGL 1.1, as supported by Microsoft Visual studio family), so names of function
 *      pointers, such as <tt>glBlendColor</tt> are present in static library, as well as in this
 *      file, causing link-time error. This is fixed by renaming the function, here
 *      <tt>glBlendColor_ptr</tt> would be used, and fixing it by macro, such as <tt>\#define
 *      glBlendColor_ptr glBlendColor</tt>.
 *
 *  @note This must not be changed for files generated with the EXPAND_GLFuncPtrName_MACRO option.
 */
#define __GL_FIX_FUNCTION_LINKING_CONFLICTS

/**
 *  @def __GL_AUTOCHECK
 *  @brief automatically checks for OpenGL errors after every OpenGL function except glGetError()
 *  @note This is only available in debug mode unless __GL_FORCE_AUTOCHECK is defined.
 */
#define __GL_AUTOCHECK

/**
 *  @def __GL_AUTOCHECK_USE_EXCEPTION
 *  @brief if defined, autocheck throws std::runtime_exception if an OpenGL error occurs
 */
//#define __GL_AUTOCHECK_USE_EXCEPTION

/**
 *  @def __GL_AUTOCHECK_USE_BREAKPOINT
 *  @brief if defined, autocheck calls user breakpoint if an OpenGL error occurs
 *  @note This is only available in windows / MSVC and in x86 mode (not x64).
 */
#define __GL_AUTOCHECK_USE_BREAKPOINT

/**
 *  @def __GL_AUTOCHECK_USE_ABORT
 *  @brief if defined, autocheck calls clib's abort() if an OpenGL error occurs
 */
//#define __GL_AUTOCHECK_USE_ABORT

/**
 *  @def __GL_AUTOCHECK_USE_EXIT
 *  @brief if defined, autocheck prints the error to stderr and calls
 *      exit(-1) if an OpenGL error occurs
 *  @note This is default behavior if nothing else is defined.
 */
//#define __GL_AUTOCHECK_USE_EXIT

/**
 *  @brief class for handling OpenGL extensions
 *
 *  Supplies means to get OpenGL function entry point, check extension support, or get
 *  OpenGL version. Functions with name n_Get*FuncPointers() returns number of function
 *  pointers that were not found (zero is success).
 */
class CGL4ExtensionHandler {
public:
    /**
     *  @brief compares OpenGL version to reference
     *
     *  Compares supplied version to current OpenGL implementation version and returns
     *      true if current version is greater or equal (supplied version is supported).
     *
     *  @param[in] n_min_major is reference minimal major version number
     *  @param[in] n_min_minor is reference minimal minor version number
     *
     *  @return Returns true in case OpenGL version is greater or equal to n_min_major.n_min_minor.
     */
    static bool b_Support_OpenGL(int n_min_major = 4, int n_min_minor = 0);

    /**
     *  @brief returns OpenGL version
     *
     *  Returns OpenGL version as (comparable) integer value.
     *
     *  @return Returns OpenGL version as major * 10 + minor (ie. for OpenGL 3.2 returns 32).
     */
    static int n_OpenGL_Version();

    /**
     *  @brief returns OpenGL version major
     *
     *  @return Returns OpenGL version as major (ie. for OpenGL 3.2 returns 3).
     */
    static int n_OpenGL_Version_Major();

    /**
     *  @brief returns OpenGL version minor
     *
     *  @return Returns OpenGL version as minor (ie. for OpenGL 3.2 returns 2).
     */
    static int n_OpenGL_Version_Minor();

    /**
     *  @brief checks extension support
     *
     *  Checks support of a given OpenGL extension, returns true if extension is supported.
     *
     *  @param[in] p_s_extension_name is null-terminated string, containing name
     *      of OpenGL extension encoded in UTF-8
     *
     *  @return Returns true in case p_s_extension_name is supported, otherwise false.
     */
    static bool b_SupportedExtension(const char *p_s_extension_name);

    /**
     *  @brief gets extension string
     *
     *  Since OpenGL 3.0 the classic extension string is no longer supported. OpenGL now
     *      returns separate names of individual extensions instead (via "indexed"
     *      ::glGetStringi() function).
     *
     *      This new behavior is supposed to prevent buffer overruns, as people always seem
     *      to be copying OpenGL extensions to fixed-size buffers, causing older software
     *      from times when extension string was shorter to fail on modern hardware.
     *
     *      This functions returns "classic" OpenGL extension string in std::string,
     *      encoded in UTF-8, names of individual extensions are separated by space (' ')
     *      character.
     *
     *  @param[out] r_s_extension_list is a place to write the extension string
     *
     *  @return Returns true on success, false on failure (not enough memory).
     */
    static bool Get_ExtensionString(std::string &r_s_extension_list);

    /**
     *  @brief checks (platform-specific) windowing system extension support
     *
     *  Checks support of a given WGL (GLX) extension, returns true if extension is supported.
     *
     *  @param[in] p_s_extension_name is null-terminated string, containing name
     *      of WGL (GLX) extension encoded in UTF-8
     *
     *  @return Returns true in case p_s_extension_name is supported, otherwise false.
     */
    static bool b_SupportedPlatformExtension(const char *p_s_extension_name);

    /**
     *  @brief gets entry points of all extensions
     *
     *  Gets pointers of function entry points for all supported extensions and OpenGL versions.
     *
     *  @return Returns number of missing functions (zero means success).
     *
     *  @note Note this is aimed at simple and test implementations without
     *      much need for error checking. it is always better to call individual
     *      functions, because in case this fails, there's no telling
     *      which extension exactly failed to initialize.
     */
    static int n_GetAllSupportedExtensionsFuncPointers();

    /**
     *  @brief gets entry points of all OpenGL core functions
     *
     *  Gets pointers of function entry points for all supported OpenGL versions.
     *
     *  @return Returns number of missing functions (zero means success).
     *
     *  @note Note this is aimed at simple and test implementations without
     *      much need for error checking. it is always better to call individual
     *      functions, because in case this fails, there's no telling
     *      which extension exactly failed to initialize.
     */
    static int n_GetAllOpenGLCoreFuncPointers();

    /**
     *  @brief gets entry points of all OpenGL core functions, up to the version specified
     *
     *  Gets pointers of function entry points for all supported OpenGL versions, up to the version specified.
     *
     *  @param[in] n_major is required OpenGL version major
     *  @param[in] n_minor is required OpenGL version minor
     *
     *  @return Returns number of missing functions (zero means success).
     *
     *  @note Note this is aimed at simple and test implementations without
     *      much need for error checking. it is always better to call individual
     *      functions, because in case this fails, there's no telling
     *      which extension exactly failed to initialize.
     */
    static int n_GetOpenGLCoreFuncPointers(int n_major = 3, int n_minor = 3);

    /**
     *  @brief gets OpenGL 1.2 core function pointers, and sets GLEH_OPENGL_1_2
     *
     *  @return Returns number of missing functions (zero means success).
     */
    static int n_GetGL12FuncPointers();

    /**
     *  @brief gets OpenGL 1.3 core function pointers, and sets GLEH_OPENGL_1_3
     *
     *  @return Returns number of missing functions (zero means success).
     */
    static int n_GetGL13FuncPointers();

    /**
     *  @brief gets OpenGL 1.4 core function pointers, and sets GLEH_OPENGL_1_4
     *
     *  @return Returns number of missing functions (zero means success).
     */
    static int n_GetGL14FuncPointers();

    /**
     *  @brief gets OpenGL 1.5 core function pointers, and sets GLEH_OPENGL_1_5
     *
     *  @return Returns number of missing functions (zero means success).
     */
    static int n_GetGL15FuncPointers();

    /**
     *  @brief gets OpenGL 2.0 core function pointers, and sets GLEH_OPENGL_2_0
     *
     *  @return Returns number of missing functions (zero means success).
     */
    static int n_GetGL20FuncPointers();

    /**
     *  @brief gets OpenGL 2.1 core function pointers, and sets GLEH_OPENGL_2_1
     *
     *  @return Returns number of missing functions (zero means success).
     */
    static int n_GetGL21FuncPointers();

    /**
     *  @brief gets OpenGL 3.0 core function pointers, and sets GLEH_OPENGL_3_0
     *
     *  @return Returns number of missing functions (zero means success).
     *
     *  @note This gets function pointers for the following core extensions as well:
     *      GL_ARB_framebuffer_object, GL_ARB_map_buffer_range and GL_ARB_vertex_array_object.
     */
    static int n_GetGL30FuncPointers();

    /**
     *  @brief gets OpenGL 3.1 core function pointers, and sets GLEH_OPENGL_3_1
     *
     *  @return Returns number of missing functions (zero means success).
     *
     *  @note This gets function pointers for the following core extensions as well:
     *      GL_ARB_uniform_buffer_object, GL_ARB_draw_instanced and GL_ARB_copy_buffer.
     */
    static int n_GetGL31FuncPointers();

    /**
     *  @brief gets OpenGL 3.2 core function pointers, and sets GLEH_OPENGL_3_2
     *
     *  @return Returns number of missing functions (zero means success).
     *
     *  @note This gets function pointers for the following core extensions as well:
     *      GL_ARB_draw_elements_base_vertex, GL_ARB_provoking_vertex, GL_ARB_sync
     *      and GL_ARB_texture_multisample.
     */
    static int n_GetGL32FuncPointers();

    /**
     *  @brief gets OpenGL 3.3 core function pointers, and sets GLEH_OPENGL_3_3
     *
     *  @return Returns number of missing functions (zero means success).
     *
     *  @note This gets function pointers from the following core extensions:
     *      GL_ARB_shader_bit_encoding, GL_ARB_blend_func_extended,
     *      GL_ARB_explicit_attrib_location, GL_ARB_occlusion_query2,
     *      GL_ARB_sampler_objects, GL_ARB_texture_rgb10_a2ui,
     *      GL_ARB_texture_swizzle, GL_ARB_timer_query, GL_ARB_instanced_arrays
     *      and GL_ARB_vertex_type_2_10_10_10_rev.
     */
    static int n_GetGL33FuncPointers();

    /**
     *  @brief gets OpenGL 4.0 core function pointers, and sets GLEH_OPENGL_4_0
     *
     *  @return Returns number of missing functions (zero means success).
     *
     *  @note This gets function pointers from the following core extensions:
     *      GL_ARB_texture_query_lod, GL_ARB_draw_indirect, GL_ARB_gpu_shader5,
     *      GL_ARB_gpu_shader_fp64 GL_ARB_shader_subroutine,
     *      GL_ARB_tessellation_shader, GL_ARB_texture_buffer_object_rgb32,
     *      GL_ARB_texture_cube_map_array, GL_ARB_texture_gather
     *      GL_ARB_transform_feedback2 and GL_ARB_transform_feedback3.
     */
    static int n_GetGL40FuncPointers();

    /**
     *  @brief gets OpenGL 4.1 core function pointers, and sets GLEH_OPENGL_4_1
     *
     *  @return Returns number of missing functions (zero means success).
     *
     *  @note This gets function pointers from the following core extensions:
     *      ARB_ES2_compatibility, ARB_get_program_binary,
     *      ARB_separate_shader_objects, ARB_shader_precision,
     *      ARB_vertex_attrib_64bit and ARB_viewport_array.
     */
    static int n_GetGL41FuncPointers();

    /**
     *  @brief gets OpenGL 4.2 core function pointers, and sets GLEH_OPENGL_4_2
     *
     *  @return Returns number of missing functions (zero means success).
     *
     *  @note This gets function pointers from the following core extensions:
     *      ARB_texture_compression_bptc, ARB_compressed_texture_pixel_storage,
     *      ARB_shader_atomic_counters, ARB_texture_storage, ARB_transform_feedback_instanced,
     *      ARB_base_instance, ARB_shader_image_load_store, ARB_conservative_depth,
     *      ARB_shading_language_420pack, ARB_internalformat_query and ARB_map_buffer_alignment.
     */
    static int n_GetGL42FuncPointers();

